<?php

namespace FiloBlu\AdyenPaymentLinkGenerator\Controller\Adminhtml\Order;

use Adyen\Payment\Helper\Data as AdyenHelper;
use FiloBlu\AdyenPaymentLinkGenerator\Helper\Data;
use Magento\Sales\Api\OrderRepositoryInterface;

class Generatelink extends \Magento\Backend\App\Action
{

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var AdyenHelper
     */
    protected $adyenHelper;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param OrderRepositoryInterface $orderRepository
     * @param AdyenHelper $adyenHelper
     * @param Data $helper
     */
    public function __construct(
        \Magento\Backend\App\Action\Context        $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        OrderRepositoryInterface                   $orderRepository,
        AdyenHelper                                $adyenHelper,
        Data                                       $helper
    )
    {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->orderRepository = $orderRepository;
        $this->adyenHelper = $adyenHelper;
        $this->helper = $helper;
    }

    /**
     * Restore order
     *
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Redirect|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $resultRedirect = $this->resultRedirectFactory->create();
        $order = $this->getOrder();

        if ($order) {
            $paymentMethod = $order->getPayment()->getMethod();
            if (preg_match(Data::PAYMENT_METHOD_ALLOW, $paymentMethod)) {
                try {
                    $client = $this->adyenHelper->initializeAdyenClient();
                    $service = $this->adyenHelper->createAdyenCheckoutService($client);
                    $storeId = $order->getStoreId();
                    $merchantAccount = $this->adyenHelper->getAdyenMerchantAccount($paymentMethod, $storeId);
                    $billingAddress = $order->getBillingAddress();
                    $shippingAddress = $order->getShippingAddress();
                    $customerId = $order->getCustomerId();
                    $expireDate = $this->helper->expireCalculation();

                    $linkParams = [
                        "reference" => $order->getIncrementId(),
                        "amount" => [
                            "value" => round($order->getGrandTotal(), 2) * 100,
                            "currency" => $order->getOrderCurrencyCode()
                        ],
                        "countryCode" => $billingAddress->getCountryId(),
                        "merchantAccount" => $merchantAccount,
                        "shopperEmail" => $order->getCustomerEmail(),
                        "expiresAt" => $expireDate->format("c"),
                        "billingAddress" => [
                            "street" => $billingAddress->getStreet()[0],
                            "postalCode" => $billingAddress->getPostcode(),
                            "city" => $billingAddress->getCity(),
                            "houseNumberOrName" => "",
                            "country" => $billingAddress->getCountryId(),
                            "stateOrProvince" => $billingAddress->getRegionCode()
                        ],
                        "deliveryAddress" => [
                            "street" => $shippingAddress->getStreet()[0],
                            "postalCode" => $shippingAddress->getPostcode(),
                            "city" => $shippingAddress->getCity(),
                            "houseNumberOrName" => "",
                            "country" => $shippingAddress->getCountryId(),
                            "stateOrProvince" => $shippingAddress->getRegionCode()
                        ]
                    ];

                    if (!is_null($customerId)) {
                        $linkParams['shopperReference'] = $this->adyenHelper->getCustomerReference($order, $customerId, $merchantAccount);
                    }

                    $hppNotAllowed = $this->helper->getPaymentLinkGeneratorHppNotAllowed();
                    if (is_array($hppNotAllowed) && !empty($hppNotAllowed)) {
                        $linkParams["blockedPaymentMethods"] = $hppNotAllowed;
                    }

                    $response = $service->paymentLinks($linkParams, []);

                    if (!empty($response) && isset($response['url'])) {
                        $this->messageManager->addSuccessMessage(__("Payment link successfully created. Check the order history for the url to send to the customer."));
                        $expiresMessage = '';
                        if (isset($response['expiresAt'])) {
                            $expiresAt = new \DateTime($response['expiresAt']);
                            $expiresAt->setTimezone(new \DateTimeZone('Europe/Rome'));
                            $expiresMessage = '<br>The link expiration date is ' . $expiresAt->format("d-m-Y H:i:s");
                        }
                        $order->addCommentToStatusHistory('Payment link created.<br>PBL data: Order ' . $order->getIncrementId() . ' / Email ' . $order->getCustomerEmail() . ' / Link ' . $response['url'] . $expiresMessage, false);

                        $payment = $order->getPayment();
                        $pblData = [];
                        if (!is_null($payment->getAdditionalInformation('pay_by_link_data'))) {
                            $pblData = json_decode($payment->getAdditionalInformation('pay_by_link_data'), true);
                        }
                        $pblData[] = [
                            'link_id' => $response['id'],
                            'notify' => 0
                        ];
                        $payment->setAdditionalInformation('pay_by_link_data', json_encode($pblData));

                        $this->orderRepository->save($order);
                    } else {
                        $this->messageManager->addSuccessMessage(__("No response. Check on Adyen if the pay by link has been created."));
                    }
                } catch (\Exception $e) {
                    $this->messageManager->addErrorMessage($e->getMessage());
                }
            } else {
                $this->messageManager->addNoticeMessage(__('Payment link not created. The original order must be placed with an adyen payment method.'));
            }
            $resultRedirect->setPath('sales/order/view', ['order_id' => $order->getId()]);
            return $resultRedirect;
        }
        $resultRedirect->setPath('sales/*/');
        return $resultRedirect;
    }

    /**
     * @return false|\Magento\Sales\Api\Data\OrderInterface
     */
    protected function getOrder()
    {
        $id = $this->getRequest()->getParam('order_id');
        try {
            $order = $this->orderRepository->get($id);
        } catch (NoSuchEntityException $e) {
            $this->messageManager->addErrorMessage(__('This order no longer exists.'));
            $this->_actionFlag->set('', self::FLAG_NO_DISPATCH, true);
            return false;
        } catch (InputException $e) {
            $this->messageManager->addErrorMessage(__('This order no longer exists.'));
            $this->_actionFlag->set('', self::FLAG_NO_DISPATCH, true);
            return false;
        }
        return $order;
    }

    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_AdyenPaymentLinkGenerator::config_filoblu_adyenpaymentlinkgenerator');
    }
}
